package archives_test

import (
	"testing"

	"gogs.uu.mdfitnesscao.com/hys/sdk"
	"gogs.uu.mdfitnesscao.com/hys/sdk/archives"
	"gogs.uu.mdfitnesscao.com/hys/sdk/utils"
)

func TestList(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.Detail("B8JB6J0d", "", "")
		list := response.Data.Detail
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("list: %v", list)
	})
}

func TestGetArchivesDataListByDate(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.ListUpdatedArchivesDataByDate("2023-09-15", "aaaaa")
		list := response.Data.List
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("list: %v", utils.JsonEncode(list))
	})
}

func TestGetArchivesListByDate(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.ListUpdatedArchivesByDate("2023-09-15")
		list := response.Data.List
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("lista :%v", list)
	})
}

func TestCheckIsArchivesFamily(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, _ := archives.CheckIsArchivesFamily([]string{
			"pWJM3jrG",
			"Ydm62mB8",
		})
		isFamily := response.Data["isFamily"]
		t.Logf("lista :%v", isFamily)
	})
}

func TestGetArchivesListByIds(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.ListArchivesById([]string{
			"Ydm660mB", "8VjdwPm4", "123",
		}, "", "", 0)
		list := response.Data.List
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("lista :%v", list)
	})
}

func TestCreateArchives(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.Create(&sdk.Archives{
			Name:     "哈哈哈哈",
			Gender:   1,
			Birthday: "2023-09-15",
			Accounts: []*sdk.Account{
				{
					Account: "18523523535",
					Type:    1,
				},
			},
		})
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("response :%v", response)
	})
}

func TestCreateArchivesOrFindByAccount(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.CreateOrFindByAccount(&sdk.Archives{
			Name:     "哈哈哈哈12",
			Gender:   1,
			Birthday: "2023-09-15",
			Accounts: []*sdk.Account{
				{
					Account: "zhangsan_hhhh",
					Type:    3,
				},
			},
		}, &sdk.ArchivesAttachForm{
			ChannelId: 3,
		}, &sdk.Account{
			Account: "zhangsan_hhhh",
			Type:    3,
		})
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("response :%v", response)
	})
}

func TestTriggerArchivesMedicalReport(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.TriggerArchivesMedicalReport("WNm0dm07")
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("response :%v", response)
	})
}

func TestGetArchivesValues(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.GetArchivesValues("yojkwRjZ", []string{"medicationStatus"})
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("response :%v", response)
	})
}

func TestPaginateArchives(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.PaginateArchives(&sdk.ArchivesPaginateQuery{
			Page:              1,
			PageSize:          100,
			RegistMechanismId: "Xwlg97rP",
			AgeEnd:            200,
		}, "", "")
		list := response.Data.List
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("list :%v", list)
	})
}

func TestListArchivesIds(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, err := archives.ListArchivesIds(nil, "Xwlg97rP", "yEw98oR0")
		list := response.Data.List
		if err != nil {
			t.Errorf("err: %v", err)
		}
		t.Logf("list :%v", list)
	})
}

func TestGetValidArchivesIdsForMechanism(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, _ := archives.GetValidArchivesIdsForMechanism([]string{
			"pWJM3jrG",
			"Ydm62mB8",
		}, "M57wVRNZ")
		isFamily := response.Data.List
		t.Logf("lista :%v", isFamily)
	})
}

func TestListChannelByIds(t *testing.T) {
	t.Run("test", func(t *testing.T) {
		sdk.InitConfig(&sdk.ClientConfig{
			ApiDomain: "https://mcapi.mdfitnesscao.com",
			AppDebug:  true,
		})
		response, _ := archives.ListChannelByIds([]int64{
			3,
		}, "")
		isFamily := response.Data.List
		t.Logf("list :%v", isFamily)
	})
}
