package pdf_template

import (
	"fmt"

	"github.com/guonaihong/gout"
	"gogs.uu.mdfitnesscao.com/hys/sdk"
	"gogs.uu.mdfitnesscao.com/hys/sdk/response"
)

// 获取模板列表
func ListByIds(ids []int64) (*sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplate]], *response.ErrCode) {
	var resp *sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplate]]
	resp, err := getReq[*sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplate]]]("/pdfTemplateService/openapi/pdfTemplate/list", gout.H{
		"ids[]": ids,
	})
	if err != nil {
		return nil, &response.ErrCode{
			Code: response.PLATFORM_ERROR,
			Msg:  err.Error(),
		}
	}
	if resp == nil {
		return nil, response.ErrPlatform
	}
	if resp.Code != 200 {
		return nil, &response.ErrCode{
			Code: resp.Code,
			Msg:  resp.Message,
		}
	}
	return resp, nil
}

// 通过机构编号获取模板列表
func ListByMechanismId(mechanismId string, authorizeType int64) (*sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplateMechanism]], *response.ErrCode) {
	var resp *sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplateMechanism]]
	resp, err := getReq[*sdk.BaseResponse[sdk.ListResponse[*sdk.PdfTemplateMechanism]]]("/pdfTemplateService/openapi/pdfTemplateMechanism/list", gout.H{
		"mechanismId": mechanismId,
		"type":        authorizeType,
	})
	if err != nil {
		return nil, &response.ErrCode{
			Code: response.PLATFORM_ERROR,
			Msg:  err.Error(),
		}
	}
	if resp == nil {
		return nil, response.ErrPlatform
	}
	if resp.Code != 200 {
		return nil, &response.ErrCode{
			Code: resp.Code,
			Msg:  resp.Message,
		}
	}
	return resp, nil
}

// postReq 发送post请求
func getReq[T any](path string, data gout.H) (T, error) {
	var res T
	fmt.Println(sdk.GetConfig())
	// 检查配置
	if sdk.GetConfig().ApiDomain == "" {
		return res, fmt.Errorf("请先配置API域名")
	}
	url := fmt.Sprintf("%s%s", sdk.GetConfig().ApiDomain, path)
	fmt.Println("请求地址 ===>", url)
	err := gout.GET(url).Debug(sdk.GetConfig().AppDebug).
		SetHeader(gout.H{}).
		SetQuery(data).
		BindJSON(&res).
		Do()
	if err != nil {
		fmt.Println("请求失败: ", err)
		return res, err
	}
	return res, nil
}
